package gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.io.File;

public class ImageFrame
        extends ShortCutFrame
        implements DisplaysPels {


    private Image image;
    private ColorModel cm = ColorModel.getRGBdefault();
    private boolean imageComesFromFile = false;


// No, you can't use bytes, cause
// Java bytes are signed. Yeah, that bites!
    private short r[][];
    private short g[][];
    private short b[][];

    private int width = 128;
    private int height = 128;

// A default file name..set to null
// to start with file open dialog.
// Set to string to start with an image.
// Use a fully qualified path name, in quotes.
    private String fileName = null;

    private MenuBar menuBar = new MenuBar();
    private Menu fileMenu = new Menu("File");

    Menu openMenu = new Menu("Open");
    MenuItem openGif_mi =
            addMenuItem(openMenu, "[g]if...");
    MenuItem default_mi =
            addMenuItem(openMenu, "[d]efault image");
    MenuItem revert_mi =
            addMenuItem(openMenu, "[r]evert");
    MenuItem fitScreen_mi =
            addMenuItem(openMenu, "fitScreen");

    public void print(String s) {
        System.out.print(s);
    }

    public void println(String s) {
        System.out.println(s);
    }

    public void fitScreen() {
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension d
                = tk.getScreenSize();
        setSize(d.width, d.height);
    }

    public void actionPerformed(ActionEvent e) {

        if (match(e, fitScreen_mi)) {
            fitScreen();
            return;
        }
        if (match(e, openGif_mi)) {
            openGif();
            return;
        }
        if (match(e, default_mi)) {
            grabNumImage();
            return;
        }
        if (match(e, revert_mi)) {
            revert();
            return;
        }
    }

    public ImageFrame(String title) {
        super(title);
        initialize();
    }


    private void initialize() {
        initMenu();
        grabNumImage();

        show();
        setSize(getWidth(), getHeight());
        repaint();
    }


    public void grabNumImage() {
        setImageComesFromFile(false);
        setWidth(NumImage.gray.length);
        setHeight(NumImage.gray[0].length);
        setR(new short[getWidth()][getHeight()]);
        setG(new short[getWidth()][getHeight()]);
        setB(new short[getWidth()][getHeight()]);
        for (int y = 0; y < getHeight(); y++) {
            for (int x = 0; x < getWidth(); x++) {
                getR()[x][y] = NumImage.gray[y][x];
                getG()[x][y] = NumImage.gray[y][x];
                getB()[x][y] = NumImage.gray[y][x];

            }
        }
        short2Image();
    }

    private void initMenu() {
        getFileMenu().add(openMenu);
        getMenuBar().add(getFileMenu());
        setMenuBar(getMenuBar());
    }

    private static Frame f = new Frame();

    public static String getReadFileName(String prompt) {
        FileDialog fd = new
                FileDialog(f, prompt);
        fd.show();
        String file_name = fd.getFile();
        if (fd.getFile() == null) return null;
        String path_name = fd.getDirectory();
        String file_string = path_name + file_name;
        //println("Opening file: "+file_string);
        fd.dispose();
        return file_string;
    }

    public static String getReadFileName() {
        return
                getReadFileName("select a file");
    }

    public void setImageResize(Image i) {
        setImage(i);

        setSize(getWidth(), getHeight());
        setVisible(true);
        repaint();
    }

    public void setImage(Image i) {
        image = i;
        waitForImage(this, i);
        setWidth(i.getWidth(this));
        setHeight(i.getHeight(this));
        image2Short();
    }

    public void setImageNoShort(Image i) {
        setImage(i);
        waitForImage(this, i);
        setWidth(i.getWidth(this));
        setHeight(i.getHeight(this));
    }

    public Image getImage() {
        waitForImage(this, image);
        return image;
    }

    public String getFileName() {
        return fileName;
    }

    private static int min = 10000;
    private static int max = -10000;
    private boolean clipped = false;

    private short clip(short i) {
        if (i < min) min = i;
        if (i > max) max = i;
        if (i < 0) {
            clipped = true;
            if (i < min) min = i;
            return 0;
        }
        if (i > 255) {
            clipped = true;
            return 255;
        }
        return i;
    }

    public void clip() {
        for (int x = 0; x < getWidth(); x++)
            for (int y = 0; y < getHeight(); y++) {
                getR()[x][y] = clip(getR()[x][y]);
                getG()[x][y] = clip(getG()[x][y]);
                getB()[x][y] = clip(getB()[x][y]);
            }
    }

    /**
       short2Image - turn 3 short arrays, r, g and b into an image.
     */
    public void short2Image() {
        clipped = false;
        setWidth(getR().length);
        setHeight(getR()[0].length);
        Toolkit tk =
                Toolkit.getDefaultToolkit();
        int pels[] = new int[getWidth() * getHeight()];
        for (int x = 0; x < getWidth(); x++)
            for (int y = 0; y < getHeight(); y++) {
                pels[x + y * getWidth()] =
                        0xff000000
                        | (getR()[x][y] << 16)
                        | (getG()[x][y] << 8)
                        | getB()[x][y];
            }
        Image i = tk.createImage(
                new
                        MemoryImageSource(
                                getWidth(),
                                getHeight(),
                                getCm(),
                                pels, 0,
                                getWidth()));
        setImageNoShort(i);
        Rectangle r = getBounds();
        repaint(0, 0, r.width, r.height);
        if (clipped)
            println(
                    "warning: clipped image. min max =" + min + "," + max);
    }

    public void pels2Image(int pels[]) {
        setWidth(getR().length);
        setHeight(getR()[0].length);
        Toolkit tk = Toolkit.getDefaultToolkit();
        Image i = tk.createImage(
                new MemoryImageSource(
                        getWidth(),
                        getHeight(),
                        getCm(),
                        pels, 0,
                        getWidth()));
        setImageNoShort(i);
        Rectangle r = getBounds();
        repaint(0, 0, r.width, r.height);
    }

    /**
       image2Short - takes the private Image instance and
       makes 3 short arrays, r, g and b.
     */
    public void image2Short() {
        if (getImage() == null) return;
        if (getWidth() < 0) return;
        if (getHeight() < 0) return;
        setR(new short[getWidth()][getHeight()]);
        setG(new short[getWidth()][getHeight()]);
        setB(new short[getWidth()][getHeight()]);

        int pels[] = new int[getWidth() * getHeight()];
        setCm(ColorModel.getRGBdefault());

        PixelGrabber grabber =
                new PixelGrabber(
                        getImage(), 0, 0,
                        getWidth(), getHeight(), pels, 0, getWidth());

        try {
            grabber.grabPixels();
        } catch (InterruptedException e) {
        }
        ;
        int i = 0;
        for (int x = 0; x < getWidth(); x++)
            for (int y = 0; y < getHeight(); y++) {
                i = x + y * getWidth();
                getB()[x][y] = (short) getCm().getBlue(pels[i]);
                getG()[x][y] = (short) getCm().getGreen(pels[i]);
                getR()[x][y] = (short) getCm().getRed(pels[i]);
            }
    }


    public void waitForImage(
            Component component,
            Image image) {
        MediaTracker tracker = new MediaTracker(component);
        try {
            tracker.addImage(image, 0);
            tracker.waitForID(0);
            if (!tracker.checkID(0))
                println("Load failure!");
        } catch (InterruptedException e) {
        }
    }

    public void openGif() {
        String fn =
                getReadFileName();
        if (fn == null) return;
        openGif(fn);
    }

    private boolean imageComesFromFtp = false;

    public void setFtpFileName(String _fn) {
        setFileName(_fn);
        setImageComesFromFile(false);
        imageComesFromFtp = true;
    }

    public void setFileName(String _fn) {
        File f = new File(_fn);
        if (f.exists()) {
            fileName = _fn;
            println(
                    "File:" + fileName +
                    "\nis " + f.length() + " bytes long");
        }
        setImageComesFromFile(true);
        imageComesFromFtp = false;

    }

    public void openGif(String fn) {
        File f = new File(fn);
        if (!f.exists()) {
            grabNumImage();
            return;
        }

        setFileName(fn);
        setImage(Toolkit.getDefaultToolkit().getImage(
                        getFileName()));
        setImageResize(getImage());
        setImageComesFromFile(true);
    }

    public void revert() {
        Toolkit tk = Toolkit.getDefaultToolkit();
        if (isImageComesFromFile()) {
            if (getFileName() == null)
                openGif();
            else {
                setImage(tk.getImage(getFileName()));
                setImageResize(getImage());
            }
        } else
            grabNumImage();

    }

    public void revertNoResize() {
        Toolkit tk = Toolkit.getDefaultToolkit();
        if (isImageComesFromFile()) {
            if (getFileName() == null)
                openGif();
            else {
                setImage(tk.getImage(getFileName()));
                setImage(getImage());
            }
        } else
            grabNumImage();

    }

    // Takes a packed RGB model and makes
    // the short arrays
    public void int2Short(int pels[]) {
        println("The width and height are"
                + getWidth() + "," + getHeight());
        setR(new short[getWidth()][getHeight()]);
        setG(new short[getWidth()][getHeight()]);
        setB(new short[getWidth()][getHeight()]);

        setCm(ColorModel.getRGBdefault());
        int i = 0;
        for (int x = 0; x < getWidth(); x++)
            for (int y = 0; y < getHeight(); y++) {
                i = x + y * getWidth();
                getB()[x][y] = (short) getCm().getBlue(pels[i]);
                getG()[x][y] = (short) getCm().getGreen(pels[i]);
                getR()[x][y] = (short) getCm().getRed(pels[i]);
            }
    }


    public static void main(String args[]) {
        ImageFrame imgFrm =
                new ImageFrame("Image Frame");
        imgFrm.openGif();
    }

    public void paint(Graphics g) {
        if (getImage() != null) {
            Rectangle r = getBounds();
            g.drawImage(getImage(), 0, 0, r.width, r.height,
                    this);
        }
    }

    public ColorModel getCm() {
        return cm;
    }

    public void setCm(ColorModel cm) {
        this.cm = cm;
    }

    public boolean isImageComesFromFile() {
        return imageComesFromFile;
    }

    public void setImageComesFromFile(boolean imageComesFromFile) {
        this.imageComesFromFile = imageComesFromFile;
    }

    public short[][] getR() {
        return r;
    }

    public void setR(short[][] r) {
        this.r = r;
    }

    public short[][] getG() {
        return g;
    }

    public void setG(short[][] g) {
        this.g = g;
    }

    public short[][] getB() {
        return b;
    }

    public void setB(short[][] b) {
        this.b = b;
    }

    public int getWidth() {
        return width;
    }

    public void setWidth(int width) {
        this.width = width;
    }

    public int getHeight() {
        return height;
    }

    public void setHeight(int height) {
        this.height = height;
    }

    public MenuBar getMenuBar() {
        return menuBar;
    }

    public void setMenuBar(MenuBar menuBar) {
        this.menuBar = menuBar;
    }

    public Menu getFileMenu() {
        return fileMenu;
    }
}